/* LATEJOIN.SMA   1.0
 * -
 * (c) 2004 Steve Dudenhoeffer
 * -
 * This plugin will help out latejoiners (and people who aren't that good) by giving them
 * experience if they are far behind their team each time they spawn.
 * The way it works:
 *
 *  Every time a player spawns, the plugin notes his level
 *  If his level is less than 75% of the average level of everybody on their team
 *  Then he is given enough EXP to bring him up to that 75%.
 * 
 *  The plugin can be configured to give players only a maximum level.
 *
 *
 *  Configuration cvars:
 *     amx_latejoin - This is the percentage of the average level which is compared with the player.
 *                    Default: 75
 *
 *     amx_maxlate - Maximum levels available through the latejoin method
 *                   ie: If you are level 0, and everybody else on the rest of the team is 10, and
 *                       this is set to 4, you will only get level 4.
 *
 *  Requires: engine, ns module
 *
 *  Some of the enums were shamelessly stolen from CheesyPeteza's Extra Levels plugin, thanks!
 */

#include <amxmodx>
#include <engine>
#include <ns>

#pragma tabsize 0

public plugin_modules()
{
  require_module("ns")
  require_module("engine")
}

enum {
  XP_LEVEL_1	=     0,
  XP_LEVEL_2	=   100,
  XP_LEVEL_3	=   250,
  XP_LEVEL_4	=   450,
  XP_LEVEL_5	=   700,
  XP_LEVEL_6	=  1000,
  XP_LEVEL_7	=  1350,
  XP_LEVEL_8	=  1750,
  XP_LEVEL_9	=  2200,
  XP_LEVEL_10	=  2700,
  XP_LEVEL_11	=  3250,
  XP_LEVEL_12	=  3850,
  XP_LEVEL_13	=  4500,
  XP_LEVEL_14	=  5200,
  XP_LEVEL_15	=  5950,
  XP_LEVEL_16	=  6750,
  XP_LEVEL_17	=  7600,
  XP_LEVEL_18	=  8500,
  XP_LEVEL_19	=  9450,
  XP_LEVEL_20	= 10450,
  XP_LEVEL_21	= 11500,
  XP_LEVEL_22	= 12600
}


new active
public plugin_init()
{
  register_plugin("Latejoin XP","1.0","Steve Dudenhoeffer")
  register_cvar("amx_latejoin","75",FCVAR_SERVER)
  register_cvar("amx_maxlate","5",FCVAR_SERVER)
  active=ns_is_combat()
}
public client_spawn(id)
{
  if (!active || get_cvar_num("mp_tournamentmode"))
    return
  // No ready roomers
  if (entity_get_int(id,EV_INT_team) == 0)
    return
  checkXP(id)
}


public client_changeteam(id,newteam,oldteam)
{
  if (!active || get_cvar_num("mp_tournamentmode"))
    return
  if (newteam > 0 && is_user_alive(id))
    checkXP(id)
}
checkXP(id)
{
  new Float:m_xp = my_xp(id)
  new MyLevel=get_level(m_xp)
  new AvgLevel=average_xp(entity_get_int(id,EV_INT_team))
  
  // This player is below the AvgLevel.  Give them XP.
  if (MyLevel < AvgLevel)
  {
    // Find difference in levels.
    new Difference = AvgLevel - MyLevel
    // Scale it
    new Float:scale = get_cvar_float("amx_latejoin")
    scale *= 0.01;
    Difference = floatround(Difference * scale,floatround_floor)
    MyLevel += Difference
    // Now change XP
    
    // Check if the level we're going to set is above the max.
    new MAX = get_cvar_num("amx_maxlate")
    if (MAX && MyLevel > MAX)
      MyLevel = MAX  // It is, lower it to the max allowed.

        
    
    new Float:xp = get_xp_for_level(MyLevel)
    // Small buffer.
    xp += 1.0
    if (xp > m_xp)
    {
      set_xp(id,xp)
      client_print(id,print_chat,"[AMXX] You have been given some experience because you're too far behind the rest of your team.")
    }
  }
  return PLUGIN_CONTINUE
}
Float:my_xp(id)
{
  return ns_get_exp(id)
}
average_xp(team)
{
  new level=0
  new count=0
  if (team == 0)
    return 0
  for (new i=1;i<=get_maxplayers();i++)
  {
    if (is_user_connected(i))
    {
      if (entity_get_int(i,EV_INT_team) == team)
      {
        level+= get_level(my_xp(i))
        count++
      }
    }
  }
  if (count > 0)
  {
    new Float:flevel = float(level) / count
    level = floatround(flevel)
    return level
  }
  return 1
}
set_xp(id,Float:experience)
{
  ns_set_exp(id,experience)
}

get_level(Float:userxp) {
  new i
	if (userxp > XP_LEVEL_21)	i=21
	else if (userxp > XP_LEVEL_20)	i=20
	else if (userxp > XP_LEVEL_19)	i=19
	else if (userxp > XP_LEVEL_18)	i=18
	else if (userxp > XP_LEVEL_17)	i=17
	else if (userxp > XP_LEVEL_16)	i=16
	else if (userxp > XP_LEVEL_15)	i=15
	else if (userxp > XP_LEVEL_14)	i=14
	else if (userxp > XP_LEVEL_13)	i=13
	else if (userxp > XP_LEVEL_12)	i=12
	else if (userxp > XP_LEVEL_11)	i=11
	else if (userxp > XP_LEVEL_10)	i=10
	else if (userxp > XP_LEVEL_9)	i=9
	else if (userxp > XP_LEVEL_8)	i=8
	else if (userxp > XP_LEVEL_7)	i=7
	else if (userxp > XP_LEVEL_6)	i=6
	else if (userxp > XP_LEVEL_5)	i=5
	else if (userxp > XP_LEVEL_4)	i=4
	else if (userxp > XP_LEVEL_3)	i=3
	else if (userxp > XP_LEVEL_2)	i=2
	else if (userxp >= XP_LEVEL_1)	i=1
	else i=1
  if (cvar_exists("amx_maxlevel")) 
  {

    if (i > get_cvar_num("amx_maxlevel"))
    {
      i=get_cvar_num("amx_maxlevel")
    }
  }
  else if (i > 10)
    i=10
  return i
	
}
Float:get_xp_for_level(level)
{
  // check it incase they're not using extra levels plugin
  if (cvar_exists("amx_maxlevel")) 
  {
    if (level > get_cvar_num("amx_maxlevel"))
    {
      level=get_cvar_num("amx_maxlevel")
    }
  }
  else
  {
    if (level > 10)
      level=10
  }
  if (level == 1) return float(XP_LEVEL_1)
  if (level == 2) return float(XP_LEVEL_2)
  if (level == 3) return float(XP_LEVEL_3)
  if (level == 4) return float(XP_LEVEL_4)
  if (level == 5) return float(XP_LEVEL_5)
  if (level == 6) return float(XP_LEVEL_6)
  if (level == 7) return float(XP_LEVEL_7)
  if (level == 8) return float(XP_LEVEL_8)
  if (level == 9) return float(XP_LEVEL_9)
  if (level == 10) return float(XP_LEVEL_10)
  if (level == 11) return float(XP_LEVEL_11)
  if (level == 12) return float(XP_LEVEL_12)
  if (level == 13) return float(XP_LEVEL_13)
  if (level == 14) return float(XP_LEVEL_14)
  if (level == 15) return float(XP_LEVEL_15)
  if (level == 16) return float(XP_LEVEL_16)
  if (level == 17) return float(XP_LEVEL_17)
  if (level == 18) return float(XP_LEVEL_18)
  if (level == 19) return float(XP_LEVEL_19)
  if (level == 20) return float(XP_LEVEL_20)
  if (level == 21) return float(XP_LEVEL_21)
  if (level == 22) return float(XP_LEVEL_22)
  return 0.0
}